function mat = cbs_bending(cbs, lambdas)
%cbs_bending Build the bending matrix for monodimensional cubic B-splines.
%
% mat = cbs_bending(cbs, lambdas)
%
% Build the complete bending matrix for monodimensional cubic B-splines.
% This file relies entirely on the initial computations given in my thesis
% (p. 108). In particular, it means that there is no performance
% improvement as there was for BBS.
%
% INPUT ARGUMENTS
%
%   cbs: B-Spline structure (as created with cbs_create).
%
%   lambdas [npts-3 vector]: regularization parameters for all
%   the knot domains. Considering that lambdas is 1-indexed and that knot 
%   interval numbers start from 0, lambdas(a) is the regularization parameter
%   for the knot domain #(a-1).
%
% OUTPUT ARGUMENTS
%
%   mat [(npts x npts) sparse matrix]: the bending matrix.
%   For a particular spline, the corresponding bending energy would be:
%   "ctrlpts * mat * ctrlpts'". Note that the transpose sign is on the
%   second "ctrlpts" because the control points are stored in a row-vector.
%
% NOTE
%  CBS is partly based on the original source code of BBS.
%
% (c)2011, Florent Brunet.

% This file is part of CBS.
%
% CBS is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% CBS is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with CBS.  If not, see <http://www.gnu.org/licenses/>.

s = (cbs.tmax - cbs.tmin) / (cbs.npts - 3);

Bb = [ 2 -3  0  1 ;
      -3  6 -3  0 ;
       0 -3  6 -3 ;
       1  0 -3  2];
Bb = Bb / (6*s^3);

mat = zeros(cbs.npts);

for j = 1:cbs.npts-3
    mat(j:j+3,j:j+3) = mat(j:j+3,j:j+3) + lambdas(j)*Bb;
end

mat = sparse(mat);
